"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createQuery = exports.generateQueries = void 0;
const method_params_type_1 = require("./method-params-type");
const typescript_1 = __importDefault(require("typescript"));
const import_utils_1 = require("../import-utils");
const services_generation_1 = require("./services-generation");
const constants_1 = require("../../../../../../common/constants");
const ts_helpers_1 = require("../../../../../../common/ts-helpers");
const ts_shorthands_1 = require("../../../../../../common/ts-shorthands");
const model_utils_1 = require("../model/model-utils");
const QUERIES_VAR_NAME = 'restQueries';
function generateQueries(restQueries, ctx) {
    // import declaration
    // import { JmixRestConnection, FetchOptions, SerializedEntity, EntitiesWithCount } from "@haulmont/jmix-rest";
    const importDec = import_utils_1.importDeclaration([constants_1.CUBA_APP_TYPE, constants_1.FETCH_OPTIONS_TYPE, constants_1.SERIALIZED_ENTITY_TYPE, constants_1.ENTITIES_WITH_COUNT_TYPE], constants_1.CUBA_APP_MODULE_SPEC);
    // compose import infos of entities, entities are used as function type parameters, i.e. 'Car' entity is used as
    // cubaApp.query<Car>("scr$Car", "allCars", {}, fetchOpts);
    const entityImportInfos = restQueries
        .map(restQuery => restQuery.entity)
        .reduce((importInfos, entityName) => {
        const entityInfo = model_utils_1.findEntityByName(entityName, ctx);
        // do not add duplicates
        if (entityInfo && !importInfos.find(ii => ii.className === entityInfo.entity.className)) {
            importInfos.push(import_utils_1.entityImportInfo(entityInfo, constants_1.ENTITIES_DIR));
        }
        return importInfos;
    }, []);
    const queriesResult = createQueries(restQueries, ctx);
    // includes composed from import infos of entities + import infos of types, used in parameters, i.e
    //
    // export type queries_FavoriteCar_allCars_params = {
    //  car: Car;
    // };
    const includes = import_utils_1.createIncludes([...queriesResult.importInfos, ...entityImportInfos]);
    return ts_helpers_1.renderTSNodes([importDec, ...includes, ...queriesResult.methodParamTypes, queriesResult.node], '\n\n');
}
exports.generateQueries = generateQueries;
function createQueries(restQueries, ctx) {
    const assignmentList = [];
    const methodParamTypes = [];
    const importInfos = [];
    const queriesByEntity = new Map();
    restQueries.forEach(q => {
        const queriesOfEntity = queriesByEntity.get(q.entity);
        if (queriesOfEntity) {
            queriesOfEntity.push(q);
        }
        else {
            queriesByEntity.set(q.entity, [q]);
        }
    });
    [...queriesByEntity.entries()].forEach(([entityName, queries]) => {
        const createItemResult = createQuery(entityName, queries, ctx);
        assignmentList.push(createItemResult.node);
        createItemResult.methodParamsTypes.forEach(mpt => methodParamTypes.push(mpt));
        importInfos.push(...createItemResult.imports);
    });
    const variableDeclaration = typescript_1.default.createVariableDeclaration(QUERIES_VAR_NAME, undefined, typescript_1.default.createObjectLiteral(assignmentList, true));
    const node = typescript_1.default.createVariableStatement([ts_shorthands_1.exportModifier()], [variableDeclaration]);
    return { node, methodParamTypes, importInfos };
}
/**
 * Call example:
 *        restQueries.Car.carsByType(cubaApp, fetchOpts?)(params?)
 *        restQueries.Car.carsByTypeCount(cubaApp, fetchOpts?)(params?)
 *        restQueries.Car.carsByTypeWithCount(cubaApp, fetchOpts?)(params?)
 *
 * Cuba REST call:
 *        cubaApp.query("mpg$Car", "carsByType", params, fetchOpts);
 *        cubaApp.queryCount("mpg$Car", "carsByType", params, fetchOpts);
 *        cubaApp.queryWithCount("mpg$Car", "carsByType", params, fetchOpts);
 */
function createQuery(entityName, queries, ctx) {
    const methodAssignments = [];
    const methodParamsTypes = [];
    const imports = [];
    const className = findClassName(entityName, ctx);
    const paramTypePrefix = 'queries_' + className;
    //'overload' queries - queries with both same entity and name
    method_params_type_1.collectMethods(queries).forEach((mwo) => {
        let paramTypeName = undefined;
        //if any of overloads has params - create special type for it
        if (mwo.methods.some(m => m.params.length > 0)) {
            const { paramTypeNode, importInfos, name } = method_params_type_1.createMethodParamsType(mwo.methods, paramTypePrefix, ctx);
            imports.push(...importInfos);
            methodParamsTypes.push(paramTypeNode);
            paramTypeName = name;
        }
        const typeArguments = [typescript_1.default.createTypeReferenceNode(className, [])];
        ['', 'Count', 'WithCount'].forEach(suffix => {
            const qName = mwo.methodName;
            let functionType = undefined;
            // Promise<EntitiesWithCount<ClassName>>
            if (suffix === 'WithCount')
                functionType = createEntitiesWithCountQueryFunctionType(className);
            // Promise<SerializedEntity<ClassName>[]>
            if (suffix === '')
                functionType = createSerializedEntityQueryFunctionType(className);
            // Promise<Number>
            if (suffix === 'Count')
                functionType = createCountQueryFunctionType();
            const cubaCallFunc = services_generation_1.cubaAppCallFunc('query' + suffix, paramTypeName, functionType, [entityName, qName], suffix !== 'Count' ? typeArguments : []);
            methodAssignments.push(typescript_1.default.createPropertyAssignment(qName + suffix, cubaCallFunc));
        });
    });
    //todo do we need to resolve entities with same names, or entity.name is unique in CUBA  ?
    const node = typescript_1.default.createPropertyAssignment(className, typescript_1.default.createObjectLiteral(methodAssignments, true));
    return { node, methodParamsTypes, imports };
}
exports.createQuery = createQuery;
/**
 * @param className name used in function type
 * @return 'Promise<SerializedEntity<ClassName>[]>'
 */
function createSerializedEntityQueryFunctionType(className) {
    const entityClassTypeNode = typescript_1.default.createTypeReferenceNode(className, []);
    const entityTypeNode = typescript_1.default.createTypeReferenceNode(constants_1.SERIALIZED_ENTITY_TYPE, [entityClassTypeNode]);
    const entityArrayTypeNode = typescript_1.default.createArrayTypeNode(entityTypeNode);
    return typescript_1.default.createTypeReferenceNode('Promise', [entityArrayTypeNode]);
}
/**
 * @param className name used in function type
 * @return 'Promise<EntitiesWithCount<ClassName>>'
 */
function createEntitiesWithCountQueryFunctionType(className) {
    const entityClassTypeNode = typescript_1.default.createTypeReferenceNode(className, []);
    const entityTypeNode = typescript_1.default.createTypeReferenceNode(constants_1.ENTITIES_WITH_COUNT_TYPE, [entityClassTypeNode]);
    return typescript_1.default.createTypeReferenceNode('Promise', [entityTypeNode]);
}
/**
 * @return 'Promise<Number>'
 */
function createCountQueryFunctionType() {
    const numberTypeNode = typescript_1.default.createTypeReferenceNode('Number', []);
    return typescript_1.default.createTypeReferenceNode('Promise', [numberTypeNode]);
}
function findClassName(entityName, ctx) {
    const entry = [...ctx.entitiesMap.entries()]
        .find(([, projEntityInfo]) => {
        return projEntityInfo.entity.name == entityName;
    });
    return entry[1].entity.className;
}
//# sourceMappingURL=queries-generation.js.map