"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const base_generator_1 = require("../../common/base-generator");
const cli_options_1 = require("../../common/cli-options");
const studio_integration_1 = require("../../common/studio/studio-integration");
const chai_1 = require("chai");
const YeomanEnvironment = require("yeoman-environment");
const generatorParams = [
    {
        caption: "Entity",
        code: "entity",
        propertyType: "ENTITY" /* ENTITY */,
        required: true
    },
    {
        caption: "Component class name",
        code: "componentName",
        propertyType: "POLYMER_COMPONENT_NAME" /* POLYMER_COMPONENT_NAME */,
        defaultValue: "Cards",
        required: true
    },
    {
        caption: "Entity view",
        code: "entityView",
        propertyType: "VIEW" /* VIEW */,
        relatedProperty: "entity",
        required: true
    }
];
const model = require.resolve('../fixtures/mpg-projectModel.json');
const answers = require('../fixtures/answers.js');
const optsConfig = {
    ...cli_options_1.commonGenerationOptionsConfig,
    answers: {
        alias: 'a',
        description: 'fulfilled params for generator to avoid interactive input in serialized JSON string',
        type: String
    }
};
class TestGenerator extends base_generator_1.BaseGenerator {
    constructor(args, options) {
        super(args, options);
    }
    async _promptOrParse() {
        return await super._promptOrParse();
    }
    _getAvailableOptions() {
        return optsConfig;
    }
    _getParams() {
        return generatorParams;
    }
    async _obtainAnswers() {
        await super._obtainAnswers();
    }
    async _obtainCubaProjectModel() {
        return super._obtainCubaProjectModel();
    }
    prompt(_questions) {
        // mock prompt answers
        return Promise.resolve({ projectInfo: 'projectInfo' });
    }
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    writing() { }
}
const opts = {
    dest: '.',
    env: new YeomanEnvironment,
    resolved: '.',
    model,
};
describe('BaseGenerator', () => {
    it('should prompt or parse', async () => {
        const gen = new TestGenerator([], {
            ...opts,
            answers: Buffer.from(JSON.stringify(answers)).toString('base64')
        });
        chai_1.expect(gen.answers).is.undefined;
        await gen._promptOrParse();
        chai_1.expect(gen.answers).not.empty;
    });
    it('should obtain answers', async () => {
        const gen = new TestGenerator([], opts);
        chai_1.expect(gen.answers).is.undefined;
        await gen._obtainAnswers();
        chai_1.expect(gen.answers.projectInfo).eq('projectInfo');
    });
    it('should fail on obtain project model if no cuba connection', (done) => {
        const gen = new TestGenerator([], { ...opts, model: undefined });
        gen
            ._obtainCubaProjectModel()
            .catch(e => {
            chai_1.expect(e.message).eq(studio_integration_1.ERR_STUDIO_NOT_CONNECTED);
            done();
        });
    });
    it('should refine answers', () => {
        const projectModel = {
            entities: [],
            enums: [],
            project: { locales: [], modulePrefix: "", name: "", namespace: "" },
            restQueries: [],
            restServices: [],
            views: []
        };
        let refined = base_generator_1.refineAnswers(projectModel, [], { intAnswer: 1.1 });
        chai_1.expect(refined).eql({ intAnswer: 1.1 });
        const generatorParams = [
            {
                caption: "Answer type integer",
                code: "intAnswer",
                propertyType: "INTEGER" /* INTEGER */,
                required: true
            },
        ];
        refined = base_generator_1.refineAnswers(projectModel, generatorParams, { intAnswer: 1 });
        chai_1.expect(refined).eql({ intAnswer: 1 });
        refined = base_generator_1.refineAnswers(projectModel, generatorParams, { intAnswer: 0 });
        chai_1.expect(refined).eql({ intAnswer: 0 });
        chai_1.expect(() => base_generator_1.refineAnswers(projectModel, generatorParams, { intAnswer: 1.1 }))
            .to.throw(`Question with code 'intAnswer' has INTEGER type and can't contain '1.1' as answer`);
        chai_1.expect(() => base_generator_1.refineAnswers(projectModel, generatorParams, { intAnswer: 'not-an-int' }))
            .to.throw(`Question with code 'intAnswer' has INTEGER type and can't contain 'not-an-int' as answer`);
        chai_1.expect(() => base_generator_1.refineAnswers(projectModel, generatorParams, { intAnswer: undefined }))
            .to.throw(`Question with code 'intAnswer' has INTEGER type and can't contain 'undefined' as answer`);
    });
});
//# sourceMappingURL=base-generator.test.js.map